/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.persistence.mock;

import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;
import org.openide.loaders.DataObject;
import it.tidalwave.metadata.Metadata.FindOption;
import it.tidalwave.metadata.Metadata.StoreOption;
import it.tidalwave.metadata.MetadataItemHolder;
import it.tidalwave.metadata.persistence.query.Criterion;
import it.tidalwave.metadata.persistence.MetadataComposite;
import it.tidalwave.metadata.persistence.MetadataProperty;
import it.tidalwave.metadata.persistence.spi.BeanAccessor;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class AssertingMetadataPersistenceMock extends MetadataPersistenceMock
  {
    public DataObject expectedDataObject;
    
    public final List<MetadataItemHolder<?>> expectedHolders = new ArrayList<MetadataItemHolder<?>>();
    
    public final List<BeanAccessor> expectedBeanAccessors = new ArrayList<BeanAccessor>();
    
    private int step = 0;
    
    public Criterion latestFindBindCountCriterion;
    
    private int findBindCountResult = -1;

    public void clear() 
      {
        step = 0;
        expectedHolders.clear();
        expectedBeanAccessors.clear();
      }
    
    @Override
    public <T> void manage (final DataObject dataObject, final MetadataItemHolder<T> holder, final StoreOption ... option)
      {
        assertTrue(expectedDataObject == dataObject);
        assertTrue(expectedHolders.get(step) == holder);
//        assertEquals(expectedItems.get(step), item);
//        assertEquals(expectedPersistenceCodeProviders.get(step), codeProvider);
        step++;
      }

    @Override
    public <T> boolean refresh (final DataObject dataObject, final MetadataItemHolder<T> holder, final FindOption ... option)
      {
        assertTrue(expectedDataObject == dataObject);
        assertTrue(expectedHolders.get(step) == holder);
        step++;
        return true;
      }  

    @Override
    public <T> void persist (final DataObject dataObject, final MetadataItemHolder<T> item, final StoreOption ... option)
      {
        assertTrue(expectedDataObject == dataObject);
        assertTrue(expectedHolders.get(step) == item);
        step++;
      }
    
    @Override
    public List<MetadataProperty> findProperties (final MetadataComposite composite) 
      {
        return Arrays.asList(new MetadataProperty(composite, "child 1"),
                             new MetadataProperty(composite, "child 2"));
      }

    @Override
    public int findDataObjectCount (final Criterion criterion) 
      {
        latestFindBindCountCriterion = null;
        
        synchronized (this)
          {
            while (findBindCountResult < 0)
              {
                try
                  {
                    wait();
                  }
                catch (InterruptedException e)
                  {
                  }
              }
            
            final int result = findBindCountResult;
            findBindCountResult = -1;
            
            return result;
          } 
      }
    
    public void setDataObjectCountResult (final int result)
      {
        synchronized (this)
          {
            this.findBindCountResult = result;
            notifyAll();
          }
      }
  }
