/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.persistence.spi;

import java.util.List;
import it.tidalwave.metadata.mock.MockBeanWithEnum;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

class BeanAccessorSupportFixture<Bean> extends BeanAccessorSupport<Bean>    
  {
    public BeanAccessorSupportFixture (final Class<Bean> itemClass, final String propertySetName)
      {
        super(itemClass, propertySetName);
      }
    
    public List<String> getPersistentPropertyNames(Bean bean)   
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public Class<?> getDefaultPropertyType(String propertyName) 
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public Object getProperty(String propertyName, Bean bean) 
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public void setProperty(Bean bean, String propertyName, Object value) 
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }
  }

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class BeanAccessorSupportTest 
  {
    private BeanAccessorSupport<MockBeanWithEnum> beanAccessor;
    
    @Before
    public void setupFixture() 
      {
        beanAccessor = new BeanAccessorSupportFixture<MockBeanWithEnum>(MockBeanWithEnum.class, "group");
      }

    @After
    public void tearDownFixture() 
      {
        beanAccessor = null;
      }

    @Test
    public void testIsPersistent() 
      {
        assertTrue(beanAccessor.isPersistent("property1"));
        assertTrue(beanAccessor.isPersistent("property2"));
      }

    @Test
    public void testGetPropertySetName() 
      {
        assertEquals("group", beanAccessor.getPropertySetName());
      }

    @Test
    public void testConvertValueToPersistence()
      throws UnknownPropertyException
      {
        assertEquals(1, beanAccessor.convertValueToPersistence("property1", 1));
        assertEquals("test", beanAccessor.convertValueToPersistence("property2", "test"));
      }

    @Test
    public void testConvertValueFromPersistence() 
      throws UnknownPropertyException
      {
        assertEquals(1, beanAccessor.convertValueFromPersistence("property1", 1));
        assertEquals("test", beanAccessor.convertValueFromPersistence("property2", "test"));
      }

    @Test
    public void testConvertPropertyNameToPersistence() 
      throws UnknownPropertyException
      {
        assertEquals("property1", beanAccessor.convertPropertyNameToPersistence("property1"));
        assertEquals("property2", beanAccessor.convertPropertyNameToPersistence("property2"));
      }

    @Test
    public void testConvertPropertyNameFromPersistence()
      throws UnknownPropertyException
      {
        assertEquals("property1", beanAccessor.convertPropertyNameFromPersistence("property1"));
        assertEquals("property2", beanAccessor.convertPropertyNameFromPersistence("property2"));
      }
  }