/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.persistence.spi;

import java.io.IOException;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import it.tidalwave.metadata.Metadata.StorageType;
import it.tidalwave.metadata.mock.MetadataItemHolderMock;
import it.tidalwave.metadata.persistence.MetadataPersistence;
import it.tidalwave.metadata.persistence.mock.AssertingMetadataPersistenceMock;
import it.tidalwave.metadata.mock.MockItem1;
import it.tidalwave.metadata.spi.MetadataItemEnhancer;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class MetadataItemPersistenceSupportTest 
  {
    private MetadataItemPersistenceSupport<MockItem1> itemPersistenceSupport;
    private FileSystem fileSystem;
    
    @Before
    public void setupFixture() 
      throws Exception 
      {
        itemPersistenceSupport = new MetadataItemPersistenceSupport(MockItem1.class, "MockItem Persistence", StorageType.EXTERNAL, "MockItem");
        fileSystem = FileUtil.createMemoryFileSystem();
      }

    @After
    public void tearDownFixture() 
      {
        itemPersistenceSupport = null;
        fileSystem = null;
      }

    @Test
    public void testStoreMetadataItem() 
      throws Exception 
      {
        final AssertingMetadataPersistenceMock persistenceProvider = 
                (AssertingMetadataPersistenceMock)MetadataPersistence.Locator.findPersistence();
        final DataObject dataObject = createDataObject("dataobject1");
        persistenceProvider.clear();
        persistenceProvider.expectedDataObject = dataObject;
//        final Metadata metadata = new MetadataMock(dataObject);
        final MockItem1 item = MetadataItemEnhancer.createEnhancedItem(new MockItem1());
        final MetadataItemHolderMock<MockItem1> holder = new MetadataItemHolderMock<MockItem1>(item);
        
        persistenceProvider.expectedHolders.add(holder); // first is called by manager()
        persistenceProvider.expectedHolders.add(holder); // second is called by refresh()
//        persistenceProvider.expectedPersistenceCodeProviders.add(exifPersistenceSourceSink.EXIF_CODE_PROVIDER);
    
        itemPersistenceSupport.storeMetadataItem(dataObject, holder);
        // TODO: this would be better done with Mockito
        // assertions are in MetadataPersistenceProviderMock
      }
    
    protected DataObject createDataObject (final String name)
      throws IOException
      {
        final FileObject fileObject = fileSystem.getRoot().createData(name);
        assertNotNull(fileObject);
        final DataObject dataObject = DataObject.find(fileObject);
        assertNotNull(dataObject);
        return dataObject;
      }
  }