/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.persistence.spi;

import it.tidalwave.metadata.persistence.mock.MetadataPersistenceMock;
import it.tidalwave.metadata.persistence.mock.BeanAccessorMock;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

class Item1Mock
  {    
  }

class Item2Mock
  {    
  }

class Item3Mock
  {    
  }

class Item4Mock
  {    
  }

class Item1MockSubClass extends Item1Mock
  {    
  }

class Item2MockSubClass extends Item2Mock
  {    
  }

class Item3MockSubClass extends Item3Mock
  {    
  }

class Item3MockSubClass2 extends Item3Mock
  {    
  }

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class MetadataPersistenceSupportTest
  {
    private static final BeanAccessor<Item1Mock> BEAN_ACCESSOR_1 = new BeanAccessorMock<Item1Mock>("group 1");
    private static final BeanAccessor<Item2Mock> BEAN_ACCESSOR_2 = new BeanAccessorMock<Item2Mock>("group 2");
    private static final BeanAccessor<Item3Mock> BEAN_ACCESSOR_3 = new BeanAccessorMock<Item3Mock>("group 3");
    private static final BeanAccessor<Item3MockSubClass2> BEAN_ACCESSOR_4 = new BeanAccessorMock<Item3MockSubClass2>("group 4");
    
    private MetadataPersistenceSupport persistence;
    
    @Before
    public void setupFixtures()
      {
        persistence = new MetadataPersistenceMock();
        persistence.register(Item1Mock.class, BEAN_ACCESSOR_1);
        persistence.register(Item2Mock.class, BEAN_ACCESSOR_2);
        persistence.register(Item3Mock.class, BEAN_ACCESSOR_3);
        persistence.register(Item3MockSubClass2.class, BEAN_ACCESSOR_4);
      }
    
    @After
    public void tearDownFixtures()
      {
        persistence = null;
      }
    
    @Test
    public void testFindBeanClassByGroupName() 
      {
        assertEquals(Item1Mock.class, persistence.findBeanClassByPropertySetName("group 1"));
        assertEquals(Item2Mock.class, persistence.findBeanClassByPropertySetName("group 2"));
        assertEquals(Item3Mock.class, persistence.findBeanClassByPropertySetName("group 3"));
        assertEquals(Item3MockSubClass2.class, persistence.findBeanClassByPropertySetName("group 4"));
      }

    @Test
    public void testFindGroupNameByBeanClass() 
      {
        assertEquals("group 1", persistence.findPropertySetNameByBeanClass(Item1Mock.class));
        assertEquals("group 2", persistence.findPropertySetNameByBeanClass(Item2Mock.class));
        assertEquals("group 3", persistence.findPropertySetNameByBeanClass(Item3Mock.class));
        
        assertEquals("group 1", persistence.findPropertySetNameByBeanClass(Item1MockSubClass.class));
        assertEquals("group 2", persistence.findPropertySetNameByBeanClass(Item2MockSubClass.class));
        assertEquals("group 3", persistence.findPropertySetNameByBeanClass(Item3MockSubClass.class));
        
        assertEquals("group 4", persistence.findPropertySetNameByBeanClass(Item3MockSubClass2.class));
        
        assertNull(persistence.findPropertySetNameByBeanClass(Item4Mock.class));
      }

    @Test
    public void testFindAccessorByClass() 
      {
        assertEquals(BEAN_ACCESSOR_1, persistence.findAccessorByClass(Item1Mock.class));
        assertEquals(BEAN_ACCESSOR_2, persistence.findAccessorByClass(Item2Mock.class));
        assertEquals(BEAN_ACCESSOR_3, persistence.findAccessorByClass(Item3Mock.class));
        
        assertEquals(BEAN_ACCESSOR_1, persistence.findAccessorByClass(Item1MockSubClass.class));
        assertEquals(BEAN_ACCESSOR_2, persistence.findAccessorByClass(Item2MockSubClass.class));
        assertEquals(BEAN_ACCESSOR_3, persistence.findAccessorByClass(Item3MockSubClass.class));
        
        assertNull(persistence.findAccessorByClass(Item4Mock.class));
        
        assertEquals(BEAN_ACCESSOR_4, persistence.findAccessorByClass(Item3MockSubClass2.class));
      }
  }