/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.persistence.spi;

import it.tidalwave.metadata.mock.MockBeanWithEnum;
import it.tidalwave.metadata.mock.MockItem1;
import it.tidalwave.metadata.mock.MockItem2;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class MistralBeanAccessorTest 
  {
    private MistralBeanAccessor<MockBeanWithEnum> beanAccessor;
    private MistralBeanAccessor<MockItem1> beanAccessor1;
    private MistralBeanAccessor<MockItem2> beanAccessor2;
    private MockBeanWithEnum bean;
    private MockItem1 mockItem1;
    private MockItem2 mockItem2;
    
    @Before
    public void setupFixture()
      {
        beanAccessor = new MistralBeanAccessor<MockBeanWithEnum>(MockBeanWithEnum.class, "group");
        beanAccessor1 = new MistralBeanAccessor<MockItem1>(MockItem1.class, "group 1");
        beanAccessor2 = new MistralBeanAccessor<MockItem2>(MockItem2.class, "group 2");
        bean = new MockBeanWithEnum();
        bean.setStringProperty("Fabrizio Giudici");
        bean.setEnumProperty(MockBeanWithEnum.MyEnum.VALUE_A);
        mockItem1 = new MockItem1();
        mockItem2 = new MockItem2();
        mockItem1.setName("The name 1");
        mockItem2.setName("The name 2");
      }
    
    @After
    public void tearDownFixture()
      {
        beanAccessor = null;
        beanAccessor1 = null;
        beanAccessor2 = null;
        bean = null;
        mockItem1 = null;
        mockItem2 = null;
      }
    
    @Test
    public void testGetPropertyType() 
      throws UnknownPropertyException
      {
        assertEquals(String.class, beanAccessor.getDefaultPropertyType("stringProperty"));
        assertEquals(Integer.class, beanAccessor.getDefaultPropertyType("enumProperty")); // enum
        assertEquals(String.class, beanAccessor1.getDefaultPropertyType("name"));
        assertEquals(String.class, beanAccessor2.getDefaultPropertyType("name"));
        assertEquals(boolean.class, beanAccessor1.getDefaultPropertyType("nameAvailable"));
        assertEquals(boolean.class, beanAccessor2.getDefaultPropertyType("nameAvailable"));
      }
    
    @Test
    public void testConvertValueToPersistence()
      throws UnknownPropertyException
      {
        assertEquals("Fabrizio Giudici", beanAccessor.convertValueToPersistence("stringProperty", "Fabrizio Giudici"));
        assertEquals(1, beanAccessor.convertValueToPersistence("enumProperty", MockBeanWithEnum.MyEnum.VALUE_A));
      }
    
    @Test
    public void testConvertValueFromPersistence()
      throws UnknownPropertyException
      {
        assertEquals("Fabrizio Giudici", beanAccessor.convertValueFromPersistence("stringProperty", "Fabrizio Giudici"));
        assertEquals(MockBeanWithEnum.MyEnum.VALUE_A, beanAccessor.convertValueFromPersistence("enumProperty", 1));
      }
    
    @Test
    public void testGetProperty()
      throws UnknownPropertyException
      {
        assertEquals("Fabrizio Giudici", beanAccessor.getProperty("stringProperty", bean));
        assertEquals(1, beanAccessor.getProperty("enumProperty", bean));
        assertEquals("The name 1", beanAccessor1.getProperty("name", mockItem1));
        assertEquals("The name 2", beanAccessor2.getProperty("name", mockItem2));
      }
    
    @Test
    public void testSetProperty()
      throws UnknownPropertyException
      {
        beanAccessor.setProperty(bean, "stringProperty", "John Doe");
        beanAccessor.setProperty(bean, "enumProperty", 2);
        assertEquals("John Doe", bean.getStringProperty());
        assertEquals(MockBeanWithEnum.MyEnum.VALUE_B, bean.getEnumProperty());
        beanAccessor1.setProperty(mockItem1, "name", "THE NAME 1");
        beanAccessor2.setProperty(mockItem2, "name", "THE NAME 2");
        assertEquals("THE NAME 1", mockItem1.getName());
        assertEquals("THE NAME 2", mockItem2.getName());
      }
    
    @Test
    public void testGetPropertySetName() 
      {
        assertEquals("group 1", beanAccessor1.getPropertySetName());
        assertEquals("group 2", beanAccessor2.getPropertySetName());
      }
    
    @Test
    public void testToString()
      {
        assertEquals("MistralBeanAccessor[group 1]", beanAccessor1.toString());
        assertEquals("MistralBeanAccessor[group 2]", beanAccessor2.toString());
      }
  }