/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.persistence.spi;

import it.tidalwave.metadata.mock.MockItem1;
import java.io.IOException;
import java.io.InputStream;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class PropertyPersistenceCodeProviderTest  
  {
    private BeanAccessor<MockItem1> codeProvider;
    
    @Before
    public void setUp() 
      throws IOException 
      {
        codeProvider = new PropertyPersistenceBeanAccessor<MockItem1>(MockItem1.class,
                                                                      "group",
                                                                      PropertyPersistenceCodeProviderTest.class,
                                                                      "TestCodeProvider.properties");
      } 

    @After
    public void tearDown() 
      {
        codeProvider = null;
      }

    @Test
    public void testGetPropertySetName() 
      {
        assertEquals("group", codeProvider.getPropertySetName());
      }

//    @Test
//    public void testFindCode() 
//      {
//        assertEquals("code1", codeProvider.findCode("property1"));
//        assertEquals("code2", codeProvider.findCode("property2"));
//        assertEquals("code3", codeProvider.findCode("property3"));
//      }

//    @Test
//    public void testFindCodeWithNull()
//      {
//        try
//          {
//            codeProvider.findCode(null); 
//            fail("IllegalArgumentException expected");
//          }
//        catch (IllegalArgumentException e)
//          {
//            // ok    
//          }
//      }
    
//    @Test
//    public void testFindPropertyName() 
//      {
//        assertEquals("property1", codeProvider.findPropertyName("code1"));
//        assertEquals("property2", codeProvider.findPropertyName("code2"));
//        assertEquals("property3", codeProvider.findPropertyName("code3"));
//      }
//    
//    @Test
//    public void testFindPropertyWithNull()
//      {
//        try
//          {
//            codeProvider.findPropertyName(null); 
//            fail("IllegalArgumentException expected");
//          }
//        catch (IllegalArgumentException e)
//          {
//            // ok    
//          }
//      }
    
    @Test
    public void testToString()
      {
        assertEquals("PropertyPersistenceCodeProvider[group, TestCodeProvider.properties]", codeProvider.toString());
      }
    
    @Test
    public void testCreateWithInputStream()
      throws IOException
      {
        final InputStream is = getClass().getResourceAsStream("TestCodeProvider.properties");
        assertNotNull(is);
        final PropertyPersistenceBeanAccessor<MockItem1> ppcp = new PropertyPersistenceBeanAccessor<MockItem1>(MockItem1.class, "group 4", is);
        assertNotNull(ppcp);
        assertEquals("property1", ppcp.findPropertyName("code1"));
        assertEquals("property2", ppcp.findPropertyName("code2"));
        assertEquals("property3", ppcp.findPropertyName("code3"));
        assertEquals("PropertyPersistenceCodeProvider[group 4, ]", ppcp.toString());
      }
    
    @Test
    public void testCreateWithWrongResource() 
      {
        try
          {
            new PropertyPersistenceBeanAccessor<MockItem1>(MockItem1.class,
                                               "group", 
                                               PropertyPersistenceCodeProviderTest.class,
                                               "WrongFile.properties");
            fail("IOException expected");
          }
        catch (IOException e)
          {
            // ok    
          }
      } 
  }
